import os
import platform
import struct
import win32api
import win32con
import win32rcparser
import win32gui
import win32ui
from pywin.mfc import dialog


class ChangeTimeDialog():
    """
    Singleton class providing the dialog to change the time

    .. note:: An experiment using the resource parser. The resource file nx_work_log.rc can be opened and modified in
              Microsoft Visual Studio
    .. document private classes
    .. automethod:: __ChangeTimeDialog
    """
    instance = None

    def __new__(cls):
        if not ChangeTimeDialog.instance:
            ChangeTimeDialog.instance = ChangeTimeDialog.__ChangeTimeDialog()
        return ChangeTimeDialog.instance

    def __getattr__(self, name):
        return getattr(self.instance, name)

    def __setattr__(self, name):
        return setattr(self.instance, name)

    class __ChangeTimeDialog(dialog.Dialog):
        """
        Internal class implementing the change time dialog
        :public
        """

        def __init__(self):
            self.hwnd = None
            rc_file = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'nx_work_log.rc')
            self.resources = win32rcparser.Parse(rc_file)
            self.minutes = 0
            self.time_changed_callback = None

        def get_time(self):
            """
            Get the current logged number of minutes
            :return: Number of minutes
            """
            if self.hwnd is not None:
                minutes = 60 * int(win32gui.GetWindowText(win32gui.GetDlgItem(self.hwnd, self.resources.ids['IDC_HOURS'])))
                minutes += int(win32gui.GetWindowText(win32gui.GetDlgItem(self.hwnd, self.resources.ids['IDC_MINUTES'])))
                self.minutes = minutes

            return self.minutes

        def set_time(self, minutes):
            """
            Set the current logged number of minutes
            :param minutes: Number of minutes
            """
            self.minutes = minutes

            if self.hwnd is not None:
                win32gui.SetWindowText(win32gui.GetDlgItem(self.hwnd, self.resources.ids['IDC_HOURS']), str(int(self.minutes / 60)))
                win32gui.SetWindowText(win32gui.GetDlgItem(self.hwnd, self.resources.ids['IDC_MINUTES']), str(self.minutes % 60))

            if self.time_changed_callback is not None:
                self.time_changed_callback(minutes)

        def set_time_changed_callback(self, time_changed_callback):
            """
            Set the callback function for time changes
            :param time_changed_callback: Callback function
            """
            self.time_changed_callback = time_changed_callback

        def show_window(self):
            """
            Show the change time dialog window
            """
            if self.hwnd is None:
                self.__create_window()
            else:
                win32gui.BringWindowToTop(self.hwnd)

            win32gui.SetWindowText(win32gui.GetDlgItem(self.hwnd, self.resources.ids['IDC_HOURS']), str(int(self.minutes / 60)))
            win32gui.SetWindowText(win32gui.GetDlgItem(self.hwnd, self.resources.ids['IDC_MINUTES']), str(self.minutes % 60))

        def __create_window(self):
            message_map = {
                win32con.WM_INITDIALOG: self.__on_init_dialog,
                win32con.WM_CLOSE: self.__on_close,
                win32con.WM_COMMAND: self.__on_command,
                win32con.WM_NOTIFY: self.__on_spin_change,
            }

            win32gui.CreateDialogIndirect(0, self.resources.dialogs['IDD_CHANGE_TIME'], 0, message_map)

        def __on_init_dialog(self, hwnd, msg, wparam, lparam):
            self.hwnd = hwnd

            # centre the dialog
            desktop = win32gui.GetDesktopWindow()
            left, top, right, bottom = win32gui.GetWindowRect(self.hwnd)
            desktop_left, desktop_top, desktop_right, desktop_bottom = win32gui.GetWindowRect(desktop)
            centre_x, centre_y = win32gui.ClientToScreen(desktop, ((desktop_right - desktop_left) // 2, (desktop_bottom - desktop_top) // 2))
            win32gui.MoveWindow(hwnd, centre_x - ((right - left) // 2), centre_y - ((bottom - top) // 2), right - left, bottom - top, 0)

        def __on_command(self, hwnd, msg, wparam, lparam):
            id = win32api.LOWORD(wparam)
            if id in [win32con.IDOK]:
                win32gui.EndDialog(hwnd, id)

        def __on_close(self, hwnd, msg, wparam, lparam):
            id = win32api.LOWORD(wparam)
            win32gui.EndDialog(hwnd, id)
            self.hwnd = None

        def __on_spin_change(self, hwnd, msg, wparam, lparam):
            id = win32api.LOWORD(wparam)

            # Read the NMUPDOWN structure
            format = 'Pqqii' if platform.architecture()[0] == '64bit' else 'Piiii'
            bytes = win32ui.GetBytes(lparam, struct.calcsize(format))
            _, _, _, _, i_delta = struct.unpack(format, bytes)

            if id == self.resources.ids['IDC_SPIN_HOURS']:
                time = self.get_time() - i_delta * 60
                self.set_time(time)
            elif id == self.resources.ids['IDC_SPIN_MINUTES']:
                time = self.get_time() - i_delta
                self.set_time(time)



import pytest
from nx_work_log.minute_timer import MinuteTimer


@pytest.fixture()
def minute_timer(minute_timer_callback):
    minute_timer = MinuteTimer(minute_timer_callback)

    yield minute_timer

    minute_timer.cancel()



import json
import os
import platform
from nx_work_log.minute_timer import MinuteTimer

if platform.system() == 'Windows':
    from nx_work_log.change_time_dialog import ChangeTimeDialog
    from nx_work_log.sys_tray_icon import SysTrayIcon
else:
    from nx_work_log.linux_change_time_dialog import ChangeTimeDialog
    from nx_work_log.linux_sys_tray_icon import SysTrayIcon

paused = True
icons = {'running': os.path.join(os.path.abspath(os.path.dirname(__file__)), 'SysTrayRunning.ico'),
         'paused': os.path.join(os.path.abspath(os.path.dirname(__file__)), 'SysTrayPaused.ico')}
minute_timer = None
my_sys_tray_icon = None
config_file_name = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'configuration.json')
configuration = {}


def show_change_time_dialog(sys_tray_icon):
    """
    This function is called when the SysTrayIcon change time menu item is selected
    :param sys_tray_icon: The SysTrayIcon object
    """
    ChangeTimeDialog().show_window()


def reset_time(sys_tray_icon):
    """
    This function is called when the SysTrayIcon change reset time menu item is selected
    :param sys_tray_icon: The SysTrayIcon object
    """
    ChangeTimeDialog().set_time(0)


def pause_clicked(sys_tray_icon):
    """
    This function is called when the SysTrayIcon change pause menu item is selected or the icon is clicked
    :param sys_tray_icon: The SysTrayIcon object
    """
    global paused
    paused = not paused
    sys_tray_icon.icon = icons['paused'] if paused else icons['running']
    sys_tray_icon.refresh_icon()


def is_paused():
    """
    This function returns if the timer is paused
    :return: Paused
    """
    return paused


def minute_timer_callback():
    """
    This is the callback function for the minute timer. It gets called every minute.
    """
    if not paused:
        dialog_time = ChangeTimeDialog().get_time()
        dialog_time += 1
        ChangeTimeDialog().set_time(dialog_time)


def change_time_dialog_time_changed_callback(minutes):
    """
    This is the callback function for the change time dialog, which keeps the time and whenever it changes due to the
    timer or user input then this function is called.
    """
    if my_sys_tray_icon is not None:
        my_sys_tray_icon.set_hover_text('{:02d}:{:02d}'.format(int(minutes / 60), minutes % 60))

    configuration['logged_minutes'] = minutes
    with open(config_file_name, 'w') as json_file:
        json.dump(configuration, json_file)


def on_startup(sys_tray_icon):
    """
    This function is called when the SysTrayIcon starts up
    :param sys_tray_icon: The SysTrayIcon object
    """
    global my_sys_tray_icon
    my_sys_tray_icon = sys_tray_icon


def on_exit(sys_tray_icon):
    """
    The exit function is called when the SystrayIcon exits
    :param sys_tray_icon: The SysTrayIcon object
    """
    minute_timer.cancel()


def exit():
    """
    This function ends the SysTray thread
    """
    SysTrayIcon().exit()


def main():
    """
    Main function creating the ChangeTimeDialog and SysTrayIcon with its menu
    """
    global minute_timer, configuration

    menu_options = [
        ('Change Time', None, show_change_time_dialog),
        ('Reset Time', None, reset_time),
        ('Pause', is_paused, pause_clicked),
    ]

    ChangeTimeDialog().set_time_changed_callback(change_time_dialog_time_changed_callback)

    # Load the logged time from the json file if it exists
    if os.path.exists(config_file_name):
        with open(config_file_name) as json_file:
            configuration = json.load(json_file)

        ChangeTimeDialog().set_time(configuration['logged_minutes'])
    else:
        configuration['logged_minutes'] = 0

    # Create a minute timer for logging the worked minutes
    minute_timer = MinuteTimer(minute_timer_callback)

    # Create the system tray icon and menus. This function is blocking.
    SysTrayIcon(icons['paused'], '{:02d}:{:02d}'.format(int(configuration['logged_minutes'] / 60), configuration['logged_minutes'] % 60),
                menu_options, call_on_startup=on_startup, on_exit=on_exit, default_menu_index=2)


if __name__ == '__main__':
    main()



import threading
import time


class MinuteTimer():
    """
    Minute Timer class calls the given callback function once every minute
    :param callback_function: Callback function to be called
    """
    def __init__(self, callback_function):
        self.callback_function = callback_function
        self.next_call = time.time()
        self.__timer()

    def cancel(self):
        """
        Cancel the timer and its internal thread
        """
        self.thread.cancel()

    def __timer(self):
        self.next_call = self.next_call + 60
        self.thread = threading.Timer(self.next_call - time.time(), self.__timer)
        self.thread.start()
        self.callback_function()



import os.path
import win32api
import win32con
import win32gui_struct

try:
    import winxpgui as win32gui
except ImportError:
    import win32gui


class SysTrayIcon(object):
    EXIT = 'EXIT'
    SPECIAL_ACTIONS = [EXIT]

    FIRST_ID = 1023

    def __init__(self, icon, hover_text, menu_options, on_exit=None, default_menu_index=None, window_class_name=None, call_on_startup=None):
        """
        The SysTrayIcon class implements an icon in the windows system tray, which allows left click on the icon for an action
        and right click for a menu
        :param icon: Full path to icon file
        :param hover_text: Text to show when hovering above icon
        :param menu_options: List of menu options and functions to call
        :param on_exit: FUnction to call when exiting
        :param default_menu_index: The menu item to select when clicking the icon
        :param window_class_name: Classname
        :param call_on_startup: Function to call on startup
        """
        self.icon = icon
        self.hover_text = hover_text
        self.on_exit = on_exit

        menu_options = menu_options + [('Exit', None, self.EXIT)]
        self._next_action_id = self.FIRST_ID
        self.menu_actions_by_id = set()
        self.menu_options = self._add_ids_to_menu_options(list(menu_options))
        self.menu_actions_by_id = dict(self.menu_actions_by_id)
        del self._next_action_id

        self.default_menu_index = (default_menu_index or 0)
        self.window_class_name = window_class_name or "SysTrayIconPy"

        message_map = {win32gui.RegisterWindowMessage("TaskbarCreated"): self.restart,
                       win32con.WM_DESTROY: self.destroy,
                       win32con.WM_COMMAND: self.command,
                       win32con.WM_USER+20: self.notify}
        if self.on_exit:
            message_map[win32con.WM_QUERYENDSESSION] = self.on_exit
        # Register the Window class.
        window_class = win32gui.WNDCLASS()
        hinst = window_class.hInstance = win32gui.GetModuleHandle(None)
        window_class.lpszClassName = self.window_class_name
        window_class.style = win32con.CS_VREDRAW | win32con.CS_HREDRAW
        window_class.hCursor = win32gui.LoadCursor(0, win32con.IDC_ARROW)
        window_class.hbrBackground = win32con.COLOR_WINDOW
        window_class.lpfnWndProc = message_map  # could also specify a wndproc.
        classAtom = win32gui.RegisterClass(window_class)
        # Create the Window.
        style = win32con.WS_OVERLAPPED | win32con.WS_SYSMENU
        self.hwnd = win32gui.CreateWindow(classAtom,
                                          self.window_class_name,
                                          style,
                                          0,
                                          0,
                                          win32con.CW_USEDEFAULT,
                                          win32con.CW_USEDEFAULT,
                                          0,
                                          0,
                                          hinst,
                                          None)
        win32gui.UpdateWindow(self.hwnd)
        self.notify_id = None
        self.refresh_icon()

        if call_on_startup is not None:
            call_on_startup(self)
        win32gui.PumpMessages()

    def _add_ids_to_menu_options(self, menu_options):
        result = []
        for menu_option in menu_options:
            option_text, option_icon, option_action = menu_option
            if callable(option_action) or option_action in self.SPECIAL_ACTIONS or option_text == "-":
                self.menu_actions_by_id.add((self._next_action_id, option_action))
                result.append(menu_option + (self._next_action_id,))
            elif non_string_iterable(option_action):
                result.append((option_text,
                               option_icon,
                               self._add_ids_to_menu_options(option_action),
                               self._next_action_id))
            else:
                print('Unknown item', option_text, option_icon, option_action)
            self._next_action_id += 1
        return result

    def refresh_icon(self, recreate=False):
        """
        Refresh the icon. To be called after updating the icon.
        :param recreate: Recreate the icon
        """
        # Try and find a custom icon
        hinst = win32gui.GetModuleHandle(None)
        if isinstance(self.icon, str):
            if os.path.isfile(self.icon):
                icon_flags = win32con.LR_LOADFROMFILE | win32con.LR_DEFAULTSIZE
                hicon = win32gui.LoadImage(hinst,
                                           self.icon,
                                           win32con.IMAGE_ICON,
                                           0,
                                           0,
                                           icon_flags)
            else:
                hicon = win32gui.LoadIcon(0, win32con.IDI_APPLICATION)
        else:
            hinst = win32api.GetModuleHandle(None)
            hicon = win32gui.LoadIcon(hinst, int(self.icon))

        if not self.notify_id or recreate:
            message = win32gui.NIM_ADD
        else:
            message = win32gui.NIM_MODIFY
        self.notify_id = (self.hwnd,
                          0,
                          win32gui.NIF_ICON | win32gui.NIF_MESSAGE | win32gui.NIF_TIP,
                          win32con.WM_USER+20,
                          hicon,
                          self.hover_text)
        win32gui.Shell_NotifyIcon(message, self.notify_id)

    def set_icon(self, filename, hover_text=None):
        """

        """
        if hover_text is not None:
            self.hover_text = hover_text
        self.icon = filename
        self.refresh_icon()

    def set_hover_text(self, hover_text):
        """

        """
        self.hover_text = hover_text
        self.refresh_icon()

    def restart(self, hwnd, msg, wparam, lparam):
        self.refresh_icon(recreate=True)

    def destroy(self, hwnd, msg, wparam, lparam):
        if self.on_exit:
            self.on_exit(self)
        nid = (self.hwnd, 0)
        win32gui.Shell_NotifyIcon(win32gui.NIM_DELETE, nid)
        win32gui.PostQuitMessage(0)  # Terminate the app.

    def notify(self, hwnd, msg, wparam, lparam):
        if lparam == win32con.WM_LBUTTONDBLCLK:
            pass
        elif lparam == win32con.WM_RBUTTONUP:
            self.show_menu()
        elif lparam == win32con.WM_LBUTTONUP:
            self.execute_menu_option(self.default_menu_index + self.FIRST_ID)
        return True

    def show_menu(self):
        menu = win32gui.CreatePopupMenu()
        self.create_menu(menu, self.menu_options)
        # win32gui.SetMenuDefaultItem(menu, 1000, 0)

        pos = win32gui.GetCursorPos()
        # See http://msdn.microsoft.com/library/default.asp?url=/library/en-us/winui/menus_0hdi.asp
        win32gui.SetForegroundWindow(self.hwnd)
        win32gui.TrackPopupMenu(menu,
                                win32con.TPM_LEFTALIGN,
                                pos[0],
                                pos[1],
                                0,
                                self.hwnd,
                                None)
        win32gui.PostMessage(self.hwnd, win32con.WM_NULL, 0, 0)

    def create_menu(self, menu, menu_options):
        for option_text, option_icon, option_action, option_id in menu_options[::-1]:
            if option_icon:  # has icon
                if not callable(option_icon):
                    option_icon = self.prep_menu_icon(option_icon)
            if option_text == "-":  # separator
                win32gui.InsertMenu(menu, 0, win32con.MF_BYPOSITION, win32con.MF_SEPARATOR, None)
            elif option_id in self.menu_actions_by_id:  # normal item
                checked = False
                if callable(option_icon):  # checkbox item
                    checked = option_icon()
                    option_icon = None  # no icon
                if checked:
                    item, extras = win32gui_struct.PackMENUITEMINFO(text=option_text, fState=win32con.MFS_CHECKED, wID=option_id)
                else:
                    item, extras = win32gui_struct.PackMENUITEMINFO(text=option_text, hbmpItem=option_icon, wID=option_id)
                win32gui.InsertMenuItem(menu, 0, 1, item)
            else:  # submenu
                submenu = win32gui.CreatePopupMenu()
                self.create_menu(submenu, option_action)
                item, extras = win32gui_struct.PackMENUITEMINFO(text=option_text, hbmpItem=option_icon, hSubMenu=submenu)
                win32gui.InsertMenuItem(menu, 0, 1, item)

    def prep_menu_icon(self, icon):
        assert os.path.exists(icon)

        # First load the icon.
        ico_x = win32api.GetSystemMetrics(win32con.SM_CXSMICON)
        ico_y = win32api.GetSystemMetrics(win32con.SM_CYSMICON)
        hicon = win32gui.LoadImage(0, icon, win32con.IMAGE_ICON, ico_x, ico_y, win32con.LR_LOADFROMFILE)

        hdcBitmap = win32gui.CreateCompatibleDC(0)
        hdcScreen = win32gui.GetDC(0)
        hbm = win32gui.CreateCompatibleBitmap(hdcScreen, ico_x, ico_y)
        hbmOld = win32gui.SelectObject(hdcBitmap, hbm)
        # Fill the background.
        brush = win32gui.GetSysColorBrush(win32con.COLOR_MENU)
        win32gui.FillRect(hdcBitmap, (0, 0, 16, 16), brush)
        # unclear if brush needs to be feed.  Best clue I can find is:
        # "GetSysColorBrush returns a cached brush instead of allocating a new
        # one." - implies no DeleteObject
        # draw the icon
        win32gui.DrawIconEx(hdcBitmap, 0, 0, hicon, ico_x, ico_y, 0, 0, win32con.DI_NORMAL)
        win32gui.SelectObject(hdcBitmap, hbmOld)
        win32gui.DeleteDC(hdcBitmap)

        return hbm.Detach()

    def command(self, hwnd, msg, wparam, lparam):
        id = win32gui.LOWORD(wparam)
        self.execute_menu_option(id)

    def exit(self):
        win32gui.DestroyWindow(self.hwnd)

    def execute_menu_option(self, id):
        menu_action = self.menu_actions_by_id[id]
        if menu_action == self.EXIT:
            win32gui.DestroyWindow(self.hwnd)
        else:
            menu_action(self)


def non_string_iterable(obj):
    try:
        iter(obj)
    except TypeError:
        return False
    else:
        return not isinstance(obj, str)



import os

if 'PYSTRAY_BACKEND' not in os.environ:
    try:
        import gi
        try:
            gi.require_version('AyatanaAppIndicator3', '0.1')
        except ValueError:
            gi.require_version('AppIndicator3', '0.1')
        os.environ['PYSTRAY_BACKEND'] = 'appindicator'
    except (ImportError, ValueError):
        os.environ['PYSTRAY_BACKEND'] = 'xorg'

from PIL import Image
import pystray


class SysTrayIcon(object):
    EXIT = 'EXIT'

    @staticmethod
    def _create_action(action_func, sys_tray_icon):
        def action(_icon, _item):
            action_func(sys_tray_icon)
        return action

    @staticmethod
    def _create_checked(check_func):
        def checked(_item):
            return check_func()
        return checked

    def _status_menu_text(self, _item):
        return self.hover_text

    def __init__(self, icon, hover_text, menu_options, on_exit=None, default_menu_index=None, window_class_name=None, call_on_startup=None):
        """
        The SysTrayIcon class implements an icon in the Linux system tray using pystray, which allows left click on the icon for an action
        and right click for a menu
        :param icon: Full path to icon file
        :param hover_text: Text to show when hovering above icon
        :param menu_options: List of menu options and functions to call
        :param on_exit: Function to call when exiting
        :param default_menu_index: The menu item to select when clicking the icon
        :param window_class_name: Unused on Linux, kept for API compatibility
        :param call_on_startup: Function to call on startup
        """
        self.icon = icon
        self.hover_text = hover_text
        self.on_exit = on_exit
        self.default_menu_index = default_menu_index or 0
        self._pystray_icon = None

        menu_items = [
            pystray.MenuItem(self._status_menu_text, None, enabled=False)
        ]
        for idx, (option_text, option_icon, option_action) in enumerate(menu_options):
            is_default = (idx == self.default_menu_index)
            if callable(option_icon):
                check_func = option_icon
                action_func = option_action
                menu_items.append(pystray.MenuItem(
                    option_text,
                    self._create_action(action_func, self),
                    checked=self._create_checked(check_func),
                    default=is_default
                ))
            else:
                action_func = option_action
                menu_items.append(pystray.MenuItem(
                    option_text,
                    self._create_action(action_func, self),
                    default=is_default
                ))

        menu_items.append(pystray.MenuItem('Exit', lambda _icon, _item: self._exit()))

        image = Image.open(self.icon)
        image.load()
        self._pystray_icon = pystray.Icon(
            "nxworklog",
            image,
            self.hover_text,
            menu=pystray.Menu(*menu_items)
        )

        def setup(tray_icon):
            tray_icon.visible = True
            if call_on_startup is not None:
                call_on_startup(self)

        self._pystray_icon.run(setup=setup)

    def refresh_icon(self):
        """
        Refresh the icon. To be called after updating the icon.
        """
        if self._pystray_icon is not None:
            image = Image.open(self.icon)
            image.load()
            self._pystray_icon.icon = image
            self._pystray_icon.title = self.hover_text

    def set_hover_text(self, hover_text):
        """
        Set the hover text for the system tray icon.
        :param hover_text: Text to display on hover
        """
        self.hover_text = hover_text
        if self._pystray_icon is not None:
            self._pystray_icon.title = hover_text
            self._pystray_icon.update_menu()

    def exit(self):
        """
        Exit the system tray icon.
        """
        self._exit()

    def _exit(self):
        if self.on_exit:
            self.on_exit(self)
        if self._pystray_icon is not None:
            self._pystray_icon.stop()



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html
from pkg_resources import get_distribution

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys
sys.path.insert(0, os.path.abspath('../'))

# -- Project information -----------------------------------------------------

project = 'nx_work_log'
copyright = '2023, Xander Hendriks'
author = 'Xander Hendriks'

release = get_distribution('nx_work_log').version.split("+")[0]

# for example take major/minor
version = release


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',  # Core library for html generation from docstrings
    'sphinx.ext.autosummary',  # Create neat summary tables
    'sphinx_rtd_theme',  # Read the Docs theme
]
autosummary_generate = True  # Turn on sphinx.ext.autosummary

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []

# This value contains a list of modules to be mocked up. This is useful when
# some external dependencies are not met at build time and break the building
# process.
autodoc_mock_imports = [
    'win32api',
    'win32con',
    'win32gui_struct',
    'win32rcparser',
    'win_app',
    'win32gui',
    'win32ui',
    'pywin'
]

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'sphinx_rtd_theme'
html_theme_path = ['_theme']

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
# html_static_path = ['_static']



import tkinter as tk
import threading


class ChangeTimeDialog():
    """
    Singleton class providing the dialog to change the time using tkinter on Linux

    .. document private classes
    .. automethod:: __ChangeTimeDialog
    """
    instance = None

    def __new__(cls):
        if not ChangeTimeDialog.instance:
            ChangeTimeDialog.instance = ChangeTimeDialog.__ChangeTimeDialog()
        return ChangeTimeDialog.instance

    def __getattr__(self, name):
        return getattr(self.instance, name)

    def __setattr__(self, name, value):
        return setattr(self.instance, name, value)

    class __ChangeTimeDialog():
        """
        Internal class implementing the change time dialog using tkinter
        """

        def __init__(self):
            self.minutes = 0
            self.time_changed_callback = None
            self._dialog_open = False

        def get_time(self):
            """
            Get the current logged number of minutes
            :return: Number of minutes
            """
            return self.minutes

        def set_time(self, minutes):
            """
            Set the current logged number of minutes
            :param minutes: Number of minutes
            """
            self.minutes = minutes

            if self.time_changed_callback is not None:
                self.time_changed_callback(minutes)

        def set_time_changed_callback(self, time_changed_callback):
            """
            Set the callback function for time changes
            :param time_changed_callback: Callback function
            """
            self.time_changed_callback = time_changed_callback

        def show_window(self):
            """
            Show the change time dialog window
            """
            if self._dialog_open:
                return

            thread = threading.Thread(target=self._show_dialog, daemon=True)
            thread.start()

        def _show_dialog(self):
            self._dialog_open = True
            try:
                root = tk.Tk()
                root.title("Change Time")
                root.resizable(False, False)

                hours_var = tk.StringVar(value=str(int(self.minutes / 60)))
                minutes_var = tk.StringVar(value=str(self.minutes % 60))

                frame = tk.Frame(root, padx=10, pady=10)
                frame.pack()

                tk.Label(frame, text="Hours:").grid(row=0, column=0)
                tk.Spinbox(frame, from_=0, to=999, textvariable=hours_var, width=5).grid(row=0, column=1, padx=5)

                tk.Label(frame, text="Minutes:").grid(row=0, column=2)
                tk.Spinbox(frame, from_=0, to=59, textvariable=minutes_var, width=5).grid(row=0, column=3, padx=5)

                def on_ok():
                    try:
                        h = int(hours_var.get())
                        m = int(minutes_var.get())
                        self.set_time(h * 60 + m)
                    except ValueError:
                        pass
                    root.destroy()

                tk.Button(frame, text="OK", command=on_ok).grid(row=1, column=0, columnspan=4, pady=(10, 0))

                root.protocol("WM_DELETE_WINDOW", on_ok)

                # Centre the window on screen
                root.update_idletasks()
                x = (root.winfo_screenwidth() - root.winfo_reqwidth()) // 2
                y = (root.winfo_screenheight() - root.winfo_reqheight()) // 2
                root.geometry("+{}+{}".format(x, y))

                root.mainloop()
            finally:
                self._dialog_open = False



import pytest
import time

timer_time = time.time()


def minute_timer_callback():
    global timer_time
    timer_time = time.time()


@pytest.mark.parametrize('minute_timer_callback', [minute_timer_callback])
def test_timer(minute_timer):
    check_time = timer_time
    counter = 0

    while counter < 2:
        if check_time != timer_time:
            assert (timer_time - check_time) == pytest.approx(60.0, 0.01), 'Incorrect timer callback timing'
            check_time = timer_time
            counter += 1
